@@ -237,6 +237,14 @@ class Metadata(object):
         """setup metadata from dict"""
         metadata = cls()
         for k, v in d.items():
+            if isinstance(v, (str,)):
+                v = {"name":k, "value":v, "dtype":None, "unit":None, "description":None}
+            elif hasattr(v, "keys"):
+                v = {"name":k, "value":v.get("value"), "dtype":v.get("dtype"),
+                     "unit":v.get("unit"), "description":v.get("description")}
+            else:
+                v = {"name":k, "value":v, "dtype":None, "unit":None, "description":None}
+
             metadata.set_attr(**v)
         return metadata
 
@@ -290,7 +298,9 @@ class Metadata(object):
     @classmethod
     def from_csv(cls, filepath):
         """create metadata from dataframe"""
-        df = pd.read_csv(filepath)
+        df = pd.read_csv(filepath, header=None)
+        df.columns = cls.ATTRIBUTEKEYS
+        df.set_index(df.name.values, inplace=True)
         metadata = cls.from_dataframe(df)
         return metadata
 
